@@ -5,237 +5,13 @@
 ;; ============================================================================
 
 (in-package "COMMON-LISP")
-(export '(nth-value function-lambda-expression defpackage
-          print-unreadable-object declaim destructuring-bind complement
+(export '(declaim destructuring-bind complement
           constantly with-standard-io-syntax with-hash-table-iterator
           read-sequence write-sequence))
 (in-package "SYSTEM")
 
 ;; ----------------------------------------------------------------------------
 
-;; X3J13 vote <123>
-
-;; Macro (nth-value n form) == (nth n (multiple-value-list form)), CLtL2 S. 184
-(defmacro nth-value (n form)
-  (if (and (integerp n) (>= n 0))
-    (if (< n (1- multiple-values-limit))
-      (if (= n 0)
-        `(PROG1 ,form)
-        (let ((resultvar (gensym)))
-          (do ((vars (list resultvar))
-               (ignores nil)
-               (i n (1- i)))
-              ((zerop i)
-               `(MULTIPLE-VALUE-BIND ,vars ,form
-                  (DECLARE (IGNORE ,@ignores))
-                  ,resultvar
-              ) )
-            (let ((g (gensym))) (push g vars) (push g ignores))
-      ) ) )
-      `(PROGN ,form NIL)
-    )
-    `(NTH ,n (MULTIPLE-VALUE-LIST ,form))
-) )
-
-;; ----------------------------------------------------------------------------
-
-;; X3J13 vote <88>
-
-;; function --> lambda expression, CLtL2 p. 682
-(defun function-lambda-expression (obj)
-  (setq obj (coerce obj 'function))
-  (cond #+FFI
-        ((eq (type-of obj) 'FFI::FOREIGN-FUNCTION)
-         (values nil nil (sys::%record-ref obj 0)))
-        ((sys::subr-info obj)
-         (values nil nil (sys::subr-info obj)))
-        ((compiled-function-p obj) ; compiled closure?
-         (let* ((name (sys::%record-ref obj 0))
-                (def (get name 'sys::definition)))
-           (values (when def (cons 'LAMBDA (cddar def))) t name)))
-        ((sys::closurep obj) ; interpreted closure?
-         (values (cons 'LAMBDA (sys::%record-ref obj 1)) ; lambda-expression without docstring
-                 (vector ; environment
-                         (sys::%record-ref obj 4) ; venv
-                         (sys::%record-ref obj 5) ; fenv
-                         (sys::%record-ref obj 6) ; benv
-                         (sys::%record-ref obj 7) ; genv
-                         (sys::%record-ref obj 8)); denv
-                 (sys::%record-ref obj 0))))) ; name
-
-;; ----------------------------------------------------------------------------
-
-;; X3J13 vote <52>
-
-;; Package-Definition und -Installation, CLtL2 S. 270
-(defmacro defpackage (packname &rest options)
-  (setq packname (string packname))
-  ;; process options:
-  (let ((size nil) ; :SIZE has been supplied
-        (documentation nil) ; :DOCUMENTATION string
-        (nickname-list '()) ; list of nicknames
-        (shadow-list '()) ; list of symbol names to shadow
-        (shadowing-list '()) ; list of pairs (symbol-name . package-name) for shadowing-import
-        (use-list '()) ; list of package-names for use-package
-        (use-default '("COMMON-LISP")) ; default for use-list
-        (case-sensitive nil) ; flag for :CASE-SENSITIVE
-        (import-list '()) ; list of (symbol-name . package-name) for import
-        (intern-list '()) ; list of symbol-names for intern
-        (symname-list '()) ; list of all symbol names specified so far
-        (export-list '())) ; liste of symbol-names for export
-    (flet ((record-symname (name)
-             (if (member name symname-list :test #'string=)
-               (error-of-type 'source-program-error
-                 (TEXT "~S ~A: the symbol ~A must not be specified more than once")
-                 'defpackage packname name)
-               (push name symname-list))))
-      (dolist (option options)
-        (if (listp option)
-          (if (keywordp (car option))
-            (case (first option)
-              (:SIZE
-               (if size
-                 (error-of-type 'source-program-error
-                   (TEXT "~S ~A: the ~S option must not be given more than once")
-                   'defpackage packname ':SIZE)
-                 (setq size t))) ; ignored
-              (:DOCUMENTATION ; ANSI-CL
-               (if documentation
-                 (error-of-type 'source-program-error
-                   (TEXT "~S ~A: the ~S option must not be given more than once")
-                   'defpackage packname ':DOCUMENTATION)
-                 (setq documentation (second option))))
-              (:NICKNAMES
-               (dolist (name (rest option))
-                 (push (string name) nickname-list)))
-              (:SHADOW
-               (dolist (name (rest option))
-                 (setq name (string name))
-                 (unless (member name shadow-list :test #'string=)
-                   (push name shadow-list)
-                   (record-symname name))))
-              (:SHADOWING-IMPORT-FROM
-               (let ((pack (string (second option))))
-                 (dolist (name (cddr option))
-                   (setq name (string name))
-                   (let ((name+pack (cons name pack)))
-                     (unless (member name+pack shadowing-list :test #'equal) ; #'string= on car and cdr
-                       (push name+pack shadowing-list)
-                       (record-symname name))))))
-              (:USE
-               (dolist (name (rest option))
-                 (push (string name) use-list))
-               (setq use-default nil))
-              (:IMPORT-FROM
-               (let ((pack (string (second option))))
-                 (dolist (name (cddr option))
-                   (setq name (string name))
-                   (let ((name+pack (cons name pack)))
-                     (unless (member name+pack import-list :test #'equal) ; #'string= on car and cdr
-                       (push name+pack import-list)
-                       (record-symname name))))))
-              (:INTERN
-               (dolist (name (rest option))
-                 (setq name (string name))
-                 (unless (member name intern-list :test #'string=)
-                   (push name intern-list)
-                   (record-symname name))))
-              (:EXPORT
-               (dolist (name (rest option))
-                 (setq name (string name))
-                 (unless (member name export-list :test #'string=)
-                   (push name export-list))))
-              (:CASE-SENSITIVE ; CLISP extension
-               (when (not (null (second option)))
-                 (setq case-sensitive t)))
-              (T (error-of-type 'source-program-error
-                   (TEXT "~S ~A: unknown option ~S")
-                   'defpackage packname (first option))))
-            (error-of-type 'source-program-error
-              (TEXT "~S ~A: invalid syntax in ~S option: ~S")
-              'defpackage packname 'defpackage option))
-          (error-of-type 'source-program-error
-            (TEXT "~S ~A: not a ~S option: ~S")
-            'defpackage packname 'defpackage option)))
-      ;; check for overlaps between intern-list and export-list:
-      (setq symname-list intern-list)
-      (mapc #'record-symname export-list))
-    ;; reverse lists and apply default values:
-    (setq nickname-list (nreverse nickname-list))
-    (setq shadow-list (nreverse shadow-list))
-    (setq shadowing-list (nreverse shadowing-list))
-    (setq use-list (or use-default (nreverse use-list)))
-    (setq import-list (nreverse import-list))
-    (setq intern-list (nreverse intern-list))
-    (setq export-list (nreverse export-list))
-    ;; produce the expansion:
-    `(EVAL-WHEN (LOAD COMPILE EVAL)
-       (SYSTEM::%IN-PACKAGE ,packname :NICKNAMES ',nickname-list :USE '()
-                            ,@(when case-sensitive `(:CASE-SENSITIVE T)))
-       ;; step 1
-       ,@(if shadow-list
-           `((SHADOW ',(mapcar #'make-symbol shadow-list) ,packname)))
-       ,@(mapcar
-          #'(lambda (pair)
-              `(SHADOWING-IMPORT-CERROR ,(car pair) ,(cdr pair) ,packname))
-          shadowing-list)
-       ;; step 2
-       ,@(if use-list `((USE-PACKAGE ',use-list ,packname)))
-       ;; step 3
-       ,@(mapcar #'(lambda (pair)
-                     `(IMPORT-CERROR ,(car pair) ,(cdr pair) ,packname))
-                 import-list)
-       ,@(mapcar #'(lambda (symname) `(INTERN ,symname ,packname))
-                 intern-list)
-       ;; step 4
-       ,@(if export-list `((INTERN-EXPORT ',export-list ,packname)))
-       ;; step 5
-       ,@(if documentation
-             `((SYS::%SET-DOCUMENTATION (find-package ,packname)
-                                        'package ,documentation)))
-       (FIND-PACKAGE ,packname))))
-
-; Hilfsfunktionen:
-(defun find-symbol-cerror (string packname calling-packname)
-  (multiple-value-bind (sym found) (find-symbol string packname)
-    (unless found
-      (cerror ; 'package-error ??
-              (TEXT "This symbol will be created.")
-              (TEXT "~S ~A: There is no symbol ~A::~A .")
-              'defpackage calling-packname packname string
-      )
-      (setq sym (intern string packname))
-    )
-    sym
-) )
-(defun shadowing-import-cerror (string packname calling-packname)
-  (shadowing-import (find-symbol-cerror string packname calling-packname)
-                    calling-packname
-) )
-(defun import-cerror (string packname calling-packname)
-  (import (find-symbol-cerror string packname calling-packname)
-          calling-packname
-) )
-(defun intern-export (string-list packname)
-  (export (mapcar #'(lambda (string) (intern string packname)) string-list)
-          packname
-) )
-
-;; ----------------------------------------------------------------------------
-
-;; X3J13 vote <40>
-
-(defmacro print-unreadable-object
-    ((&whole args object stream &key type identity) &body body)
-  (declare (ignore object stream type identity))
-  `(SYSTEM::WRITE-UNREADABLE
-     ,(if body `(FUNCTION (LAMBDA () ,@body)) 'NIL)
-     ,@args
-   )
-)
-
-;; ----------------------------------------------------------------------------
-
 ;; X3J13 vote <144>
 
 (defmacro declaim (&rest decl-specs)
@@ -248,15 +24,18 @@
 
 ;; X3J13 vote <64>
 
-(defmacro destructuring-bind (lambdalist form &body body)
+(defmacro destructuring-bind (&whole whole-form
+                              lambdalist form &body body)
   (multiple-value-bind (body-rest declarations) (system::parse-body body)
     (if declarations (setq declarations `((DECLARE ,@declarations))))
-    (let ((%arg-count 0) (%min-args 0) (%restp nil)
+    (let ((%whole-form whole-form)
+          (%arg-count 0) (%min-args 0) (%restp nil) (%ignored nil)
           (%let-list nil) (%keyword-tests nil) (%default-form nil))
       (analyze1 lambdalist '<DESTRUCTURING-FORM> 'destructuring-bind '<DESTRUCTURING-FORM>)
       (let ((lengthtest (make-length-test '<DESTRUCTURING-FORM> 0))
             (mainform `(LET* ,(nreverse %let-list)
                          ,@declarations
+                         ,@(if %ignored `((DECLARE (IGNORE ,%ignored))))
                          ,@(nreverse %keyword-tests)
                          ,@body-rest
            ))          )
@@ -271,7 +50,7 @@
         `(LET ((<DESTRUCTURING-FORM> ,form)) ,mainform)
 ) ) ) )
 
-(defun destructuring-error (destructuring-form min.max)
+(defun destructuring-error (destructuring-form min.max) ; ABI
   (let ((min (car min.max))
         (max (cdr min.max)))
     (error-of-type 'program-error
@@ -279,6 +58,13 @@
       max (eql min max) min max destructuring-form
 ) ) )
 
+;; Like destructuring-bind, except that it works only with ordinary
+;; lambda-lists and generates more efficient code.
+;; Compare   (disassemble #'(lambda (l) (destructuring-bind (&optional (o 'def)) l o)))
+;; with      (disassemble #'(lambda (l) (simple-destructuring-bind (&optional (o 'def)) l o)))
+(defmacro simple-destructuring-bind (lambdalist form &body body)
+  `(APPLY #'(LAMBDA ,lambdalist ,@body) ,form))
+
 ;; ----------------------------------------------------------------------------
 
 ;; X3J13 vote <87>
@@ -297,9 +83,11 @@
 
 ;; part of X3J13 vote <40>
 
-(defconstant *common-lisp-user-package* (find-package "COMMON-LISP-USER"))
+(defconstant *common-lisp-user-package* (find-package "COMMON-LISP-USER")) ; ABI
 
 (defmacro with-standard-io-syntax (&body body)
+  ;; ANSI CL does not allow declarations at the beginning of the body, but
+  ;; we do, as an extension.
   (multiple-value-bind (body-rest declarations) (SYSTEM::PARSE-BODY body)
     ;; It would be possible to put all these bindings into a single function,
     ;; but this would force variables into closures.
@@ -327,7 +115,6 @@
            (SYSTEM::*PRIN-STREAM*       NIL) ; CLISP specific
            (SYSTEM::*PRIN-LINELENGTH*   79)  ; CLISP specific
            (SYSTEM::*PRIN-LINE-PREFIX*  NIL) ; CLISP specific
-           (CUSTOM::*PRINT-SYMBOLS-LONG* NIL) ; CLISP specific
            (COMPILER::*LOAD-FORMS*      NIL) ; CLISP specific
            ;; reader variables:
            (*READ-BASE*                 10)
@@ -355,57 +142,60 @@
 
 ;; ANSI-CL
 
-(defmacro lambda (&whole whole lambdalist &body body)
+(defmacro lambda (&whole whole-form lambdalist &body body)
   (declare (ignore lambdalist body))
-  `(FUNCTION ,whole))
+  `(FUNCTION ,whole-form))
 
 ;; ----------------------------------------------------------------------------
 
 ;; Make GET-MACRO-CHARACTER work on dispatch macro characters.
-(let ((vector '#()))
-  (declare (compile))
-  ; This code must be in accordance with io.d:read_macro().
-  (defun dispatch-reader (stream ch)
-    (let ((arg 0)
-          subch)
-      (let ((flag nil))
-        (loop
-          (let ((nextch (read-char stream nil nil)))
-            (unless nextch
-              (error-of-type 'end-of-file
-                :stream stream
-                (TEXT "~S: input stream ~S ends within read macro beginning to ~S")
-                'read stream ch
-            ) )
-            (unless (characterp nextch)
-              (error-of-type 'stream-error
-                :stream stream
-                (TEXT "~S from ~S: character read should be a character: ~S")
-                'read stream ch
-            ) )
-            (unless (char<= #\0 nextch #\9)
-              (setq subch nextch)
-              (return)
-            )
-            (setq arg (+ (* 10 arg) (digit-char-p nextch)))
-            (setq flag t)
-        ) )
-        (unless flag (setq arg nil))
-      )
-      (let* ((subc (char-upcase subch))
-             (macrodef
-               (if (< (char-code subc) #x100)
-                 (svref vector (char-code subc))
-                 (gethash subc (svref vector #x100))
-            )) )
-        (unless macrodef
-          (error-of-type 'stream-error
-            :stream stream
-            (TEXT "~S from ~S: After ~S is ~S an undefined dispatch macro character")
-            'read stream ch subch
-        ) )
-        (funcall macrodef stream subch arg)
-  ) ) )
+(let ((vector
+        (let ((vector '#()))
+          (declare (compile))
+          ; This code must be in accordance with io.d:read_macro().
+          (defun dispatch-reader (stream ch)
+            (let ((arg 0)
+                  subch)
+              (let ((flag nil))
+                (loop
+                  (let ((nextch (read-char stream nil nil)))
+                    (unless nextch
+                      (error-of-type 'end-of-file
+                        :stream stream
+                        (TEXT "~S: input stream ~S ends within read macro beginning to ~S")
+                        'read stream ch
+                    ) )
+                    (unless (characterp nextch)
+                      (error-of-type 'stream-error
+                        :stream stream
+                        (TEXT "~S from ~S: character read should be a character: ~S")
+                        'read stream ch
+                    ) )
+                    (unless (char<= #\0 nextch #\9)
+                      (setq subch nextch)
+                      (return)
+                    )
+                    (setq arg (+ (* 10 arg) (digit-char-p nextch)))
+                    (setq flag t)
+                ) )
+                (unless flag (setq arg nil))
+              )
+              (let* ((subc (char-upcase subch))
+                     (macrodef
+                       (if (< (char-code subc) #x100)
+                         (svref vector (char-code subc))
+                         (gethash subc (svref vector #x100))
+                    )) )
+                (unless macrodef
+                  (error-of-type 'stream-error
+                    :stream stream
+                    (TEXT "~S from ~S: After ~S is ~S an undefined dispatch macro character")
+                    'read stream ch subch
+                ) )
+                (funcall macrodef stream subch arg)
+          ) ) )
+          vector
+     )) )
   (let ((vector-index
           (do ((i 0 (1+ i)))
                (nil)
@@ -591,13 +381,13 @@
 ) )
 
 #-BASE-CHAR=CHARACTER
-(defun base-char-designator-p (obj)
+(defun base-char-designator-p (obj) ; ABI
   (base-char-p (char (coerce obj 'string) 0))
 )
 
 ;(defun class-designator-p (sym &aux f)
 ;  (and (setq f (get sym 'CLOS::CLOSCLASS))
-;       (clos::class-p f)
+;       (clos::defined-class-p f)
 ;       (eq (clos:class-name f) sym)
 ;) )
 
@@ -612,8 +402,7 @@
     ;; check-value will be defined in condition.lisp
     (setq name (check-value
                 nil (make-condition 'simple-type-error
-                      :format-control (TEXT "~S: ~S should be a ~S")
-                      :format-arguments
-                      (list 'define-hash-table-test name 'symbol)
+                      :format-control (TEXT "~S: ~S should be a symbol")
+                      :format-arguments (list 'define-hash-table-test name)
                       :datum name :expected-type 'symbol))))
   `(progn (setf (get ',name 'hash-table-test) (cons #',test #',hash)) ',name))
